@@ -16,6 +16,7 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <signal.h>
+#include <ctype.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/vfs.h>
@@ -145,6 +146,33 @@ int id2name(int id, int qtype, char *buf)
 		return gid2group(id, buf);
 }
 
+/*
+ *	Parse /etc/nsswitch.conf and return type of default passwd handling
+ */
+int passwd_handling(void)
+{
+	FILE *f;
+	char buf[1024], *colpos, *spcpos;
+	int ret = PASSWD_FILES;
+
+	if (!(f = fopen("/etc/nsswitch.conf", "r")))
+		return PASSWD_FILES;	/* Can't open nsswitch.conf - fallback on compatible mode */
+	while (fgets(buf, sizeof(buf), f)) {
+		if (strncmp(buf, "passwd:", 7))	/* Not passwd entry? */
+			continue;
+		for (colpos = buf+7; isspace(*colpos); colpos++);
+		if (!*colpos)	/* Not found any type of handling? */
+			break;
+		for (spcpos = colpos; !isspace(*spcpos) && *spcpos; spcpos++);
+		*spcpos = 0;
+		if (!strcmp(colpos, "db") || !strcmp(colpos, "nis") || !strcmp(colpos, "nis+"))
+			ret = PASSWD_DB;
+		break;
+	}
+	fclose(f);
+	return ret;
+}
+
 /*
  *	Convert quota format name to number
  */
@@ -256,6 +284,24 @@ void time2str(time_t seconds, char *buf, int flags)
 	}
 }
 
+/*
+ * Convert number with unit to time in seconds
+ */
+int str2timeunits(time_t num, char *unit, time_t *res)
+{
+	if (memcmp(unit, "second", 6) == 0)
+		*res = num;
+	else if (memcmp(unit, "minute", 6) == 0)
+		*res = num * 60;
+	else if (memcmp(unit, "hour", 4) == 0)
+		*res = num * 60 * 60;
+	else if (memcmp(unit, "day", 3) == 0)
+		*res = num * 24 * 60 * 60;
+	else
+		return -1;
+	return 0;
+}
+
 /*
  * Convert number in quota blocks to some nice short form for printing
  */
@@ -300,7 +346,6 @@ static int hasxfsquota(struct mntent *mnt, int type)
 	int ret = 0;
 	u_int16_t sbflags;
 	struct xfs_mem_dqinfo info;
-	int nonrootfs = strcmp(mnt->mnt_dir, "/");
 	const char *dev = get_device_name(mnt->mnt_fsname);
 
 	if (!dev)
@@ -313,12 +358,19 @@ static int hasxfsquota(struct mntent *mnt, int type)
 			ret = 1;
 		else if (type == GRPQUOTA && (info.qs_flags & XFS_QUOTA_GDQ_ACCT))
 			ret = 1;
-		else if (nonrootfs)
+#ifdef XFS_ROOTHACK
+		/*
+		 * Old XFS filesystems (up to XFS 1.2 / Linux 2.5.47) had a
+		 * hack to allow enabling quota on the root filesystem without
+		 * having to specify it at mount time.
+		 */
+		else if (strcmp(mnt->mnt_dir, "/"))
 			ret = 0;
 		else if (type == USRQUOTA && (sbflags & XFS_QUOTA_UDQ_ACCT))
 			ret = 1;
 		else if (type == GRPQUOTA && (sbflags & XFS_QUOTA_GDQ_ACCT))
 			ret = 1;
+#endif /* XFS_ROOTHACK */
 	}
 	free((char *)dev);
 	return ret;
