@@ -14,7 +14,7 @@
 #define MAXNAMELEN 64		/* Maximal length of user/group name */
 #define MAXTIMELEN 40		/* Maximal length of time string */
 #define MAXNUMLEN 32		/* Maximal length of number */
-#define MAXMNTPOINTS 128	/* Maximal number of mountpoints with quota */
+#define MAXMNTPOINTS 256	/* Maximal number of mountpoints with quota */
 
 /* Flags for formatting time */
 #define TF_ROUND 0x1		/* Should be printed time rounded? */
@@ -58,6 +58,12 @@ int gid2group(gid_t, char *);
 /* Convert id to user/group name */
 int id2name(int id, int qtype, char *buf);
 
+/* Possible default passwd handling */
+#define PASSWD_FILES 0
+#define PASSWD_DB 1
+/* Parse /etc/nsswitch.conf and return type of default passwd handling */
+int passwd_handling(void);
+
 /* Convert quota format name to number */
 int name2fmt(char *str);
 
@@ -76,6 +82,9 @@ void difftime2str(time_t, char *);
 /* Convert time to printable form */
 void time2str(time_t, char *, int);
 
+/* Convert number and units to time in seconds */
+int str2timeunits(time_t, char *, time_t *);
+
 /* Convert number in quota blocks to short printable form */
 void space2str(qsize_t, char *, int);
 
