@@ -12,7 +12,7 @@
  *          changes for new utilities by Jan Kara <jack@suse.cz>
  *          patches by Jani Jaakkola <jjaakkol@cs.helsinki.fi>
  *
- * Version: $Id: rquota_svc.c,v 1.10 2002/03/27 16:21:26 jkar8572 Exp $
+ * Version: $Id: rquota_svc.c,v 1.13 2002/11/28 22:02:04 jkar8572 Exp $
  *
  *          This program is free software; you can redistribute it and/or
  *          modify it under the terms of the GNU General Public License as
@@ -31,6 +31,7 @@
 #include <memory.h>
 #include <unistd.h>
 #include <getopt.h>
+#include <signal.h>
 #ifdef HOSTS_ACCESS
 #include <tcpd.h>
 #include <netdb.h>
@@ -40,6 +41,9 @@
 #define SIG_PF void(*)(int)
 #endif
 
+extern int svctcp_socket (u_long __number, int __reuse);
+extern int svcudp_socket (u_long __number, int __reuse);
+
 #include "pot.h"
 #include "common.h"
 #include "rquota.h"
@@ -52,24 +56,35 @@ char *progname;
  */
 struct authunix_parms *unix_cred;
 
-int disable_setquota=0;            /* Disables setquota rpc */
+int disable_setquota=1;            /* Disables setquota rpc */
 int disable_daemon=0;              /* Disable daemon() call */
 
 static struct option options[]= {
 	{ "version", 0, NULL, 'V' },
 	{ "help", 0, NULL, 'h' },
 	{ "foreground", 0 , NULL, 'F' },
+#ifdef RPC_SETQUOTA
 	{ "no-setquota", 0 , NULL, 's' },
+	{ "setquota", 0, NULL, 'S' },
+#endif
 	{ NULL, 0, NULL , 0 }
 };
 
 static void show_help(void)
 {
+#ifdef RPC_SETQUOTA
 	errstr(_("Usage: %s [options]\nOptions are:\n\
  -h --help         shows this text\n\
  -V --version      shows version information\n\
  -F --foreground   starts the quota service in foreground\n\
- -s --no-setquota  disables remote calls to setquota\n"), progname);
+ -s --no-setquota  disables remote calls to setquota (default)\n\
+ -S --setquota     enables remote calls to setquota\n"), progname);
+#else
+	errstr(_("Usage: %s [options]\nOptions are:\n\
+ -h --help         shows this text\n\
+ -V --version      shows version information\n\
+ -F --foreground   starts the quota service in foreground\n"), progname);
+#endif
 }
 
 static void parse_options(int argc, char **argv)
@@ -87,7 +102,10 @@ static void parse_options(int argc, char **argv)
 			case 'V': version(); exit(0);
 			case 'h': show_help(); exit(0);
 			case 'F': disable_daemon = 1; break;
+#ifdef RPC_SETQUOTA
 			case 's': disable_setquota = 1; break;
+			case 'S': disable_setquota = 0; break;
+#endif
 			default:
 				errstr(_("Unknown option '%c'.\n"), opt);
 				show_help();
@@ -336,9 +354,18 @@ static void rquotaprog_2(struct svc_req *rqstp, register SVCXPRT * transp)
 	return;
 }
 
+static void
+unregister (int sig)
+{
+	(void)pmap_unset(RQUOTAPROG, RQUOTAVERS);
+	(void)pmap_unset(RQUOTAPROG, EXT_RQUOTAVERS);
+}
+
 int main(int argc, char **argv)
 {
 	register SVCXPRT *transp;
+	struct sigaction sa;
+	int sock;
 
 	gettexton();
 	progname = basename(argv[0]);
@@ -348,7 +375,18 @@ int main(int argc, char **argv)
 	(void)pmap_unset(RQUOTAPROG, RQUOTAVERS);
 	(void)pmap_unset(RQUOTAPROG, EXT_RQUOTAVERS);
 
-	transp = svcudp_create(RPC_ANYSOCK);
+	sa.sa_handler = SIG_IGN;
+	sa.sa_flags = 0;
+	sigemptyset(&sa.sa_mask);
+	sigaction(SIGCHLD, &sa, NULL);
+
+	sa.sa_handler = unregister;
+	sigaction(SIGHUP, &sa, NULL);
+	sigaction(SIGINT, &sa, NULL);
+	sigaction(SIGTERM, &sa, NULL);
+
+	sock = svcudp_socket(RQUOTAPROG, 1);
+	transp = svcudp_create(sock == -1 ? RPC_ANYSOCK : sock);
 	if (transp == NULL) {
 		errstr(_("cannot create udp service.\n"));
 		exit(1);
@@ -362,7 +400,8 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
-	transp = svctcp_create(RPC_ANYSOCK, 0, 0);
+	sock = svctcp_socket(RQUOTAPROG, 1);
+	transp = svctcp_create(sock == -1 ? RPC_ANYSOCK : sock, 0, 0);
 	if (transp == NULL) {
 		errstr(_("cannot create tcp service.\n"));
 		exit(1);
